
#include <Adafruit_NeoPixel.h>

#define DEBUG true
#define PIXEL_NUM 16 // Number of pixels in the NeoPixel ring
#define PIXEL_PIN 6  // The digital pin connected to the NeoPixel ring
#define SENSOR_PIN 2 // The digital pin connected to the IR sensor
#define MIN_RPM 0    // Minimum RPM value for scaling
#define MAX_RPM 800  // Maximum RPM value for scaling

Adafruit_NeoPixel pixel = Adafruit_NeoPixel(PIXEL_NUM, PIXEL_PIN, NEO_GRB + NEO_KHZ800);

const byte PulsesPerRevolution = 8;
const unsigned long ZeroTimeout = 1000000; // Timeout for zero RPM calculation
volatile unsigned long LastTimeWeMeasured;
volatile unsigned long PeriodBetweenPulses = ZeroTimeout + 1000;
unsigned long FrequencyRaw;
unsigned long RPM;
unsigned long LastTimeCycleMeasure = LastTimeWeMeasured;
unsigned long CurrentMicros = micros();
unsigned int ZeroDebouncingExtra;

void setup() {
  pixel.begin();
  pixel.show(); // Initialize all pixels to 'off'
  attachInterrupt(digitalPinToInterrupt(SENSOR_PIN), pulseEvent, RISING);
  if (DEBUG) {
    Serial.begin(115200);
  }
}

void loop() {
  LastTimeCycleMeasure = LastTimeWeMeasured;
  CurrentMicros = micros();
  if (CurrentMicros < LastTimeCycleMeasure) {
    LastTimeCycleMeasure = CurrentMicros;
  }

  if (PeriodBetweenPulses > ZeroTimeout - ZeroDebouncingExtra || CurrentMicros - LastTimeCycleMeasure > ZeroTimeout - ZeroDebouncingExtra) {
    FrequencyRaw = 0;
    ZeroDebouncingExtra = 2000;
  } else {
    FrequencyRaw = 10000000000 / PeriodBetweenPulses;
    ZeroDebouncingExtra = 0;
  }

  RPM = FrequencyRaw / PulsesPerRevolution * 60 / 10000;

  if (DEBUG) {
    Serial.print("RPM: ");
    Serial.println(RPM);
  }

  int pixelsToLight = map(constrain(RPM, MIN_RPM, MAX_RPM), MIN_RPM, MAX_RPM, 0, PIXEL_NUM);
  for (int i = 0; i < PIXEL_NUM; i++) {
    if (i < pixelsToLight) {
      pixel.setPixelColor(i, pixel.Color(255, 0, 0)); // Red color
    } else {
      pixel.setPixelColor(i, pixel.Color(0, 0, 0)); // Off
    }
  }
  pixel.show();
}

void pulseEvent() {
  unsigned long currentTime = micros();
  PeriodBetweenPulses = currentTime - LastTimeWeMeasured;
  LastTimeWeMeasured = currentTime;
}
